/*
 * Copyright 2000 by Jean-Claude Christophe <jch@oleane.net>
 *                                          <jch@bsdfr.org>
 *
 * Home page:  http://www.bsdfr.org/projects/tac_bsd/
 *
 * conf.c  Some functions for parse configuration file
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <syslog.h>

#include "tac_bsd.h"
#include "conf.h"

/* global part of configuration */

char *secret;       /* key of the tac_+ server */
int  slog;          /* 0: syslog disable; 1: syslog enable */
int  slog_flty;     /* the syslog facility */
int  slog_lvl;      /* the syslog level */
int  _syslog;       /* the definitive syslog (facility | level) */
char *premotd;      /* file to display before asking username */
char *motdpass;     /* file to display when authentication succeded */
char *motdfail;     /* file to display when authentication failed */
char *shell;        /* list of shell to use */
int  maxtry;        /* you have 3 chance by default; */
int  maxsession;    /* sessions maximum by default */
int  maxlogintime;  /* timeout for each login attempt (0 will disable it) */
char *directory;    /* directory to use when logged in */
int  passwd_care;   /* 0: we take care about the the /etc/passwd file 1: ignore it */
char *server;       /* This is the server we'll connect to */         

struct s_options {
  char *options;
  char seq;
} line_conf[] = {{"server ",           1}, \
		 {"secret ",           2}, \
		 {"syslog ",           3}, \
		 {"syslog_facility ",  4}, \
		 {"syslog_level ",     5}, \
		 {"premotd ",          6}, \
		 {"motdpass ",         7}, \
		 {"motdfail ",         8}, \
		 {"shell ",            9}, \
		 {"maxtry ",          10}, \
		 {"maxsession ",      11}, \
		 {"directory ",       12}, \
		 {"umask ",           13}, \
		 {"passwd_care ",     14}, \
		 {"maxlogintime ",    15}, \
		 {"uid ",             16}, \
		 {"gid ",             17}};

/* syslog part of configuration */
struct s_slog_flty {
  char *facility_in_conf;
  int facility;
} sslog_flty[] = {{"local0", LOG_LOCAL0}, \
		  {"local1", LOG_LOCAL1}, \
		  {"local2", LOG_LOCAL2}, \
		  {"local3", LOG_LOCAL3}, \
		  {"local4", LOG_LOCAL4}, \
		  {"local5", LOG_LOCAL5}, \
		  {"local6", LOG_LOCAL6}, \
		  {"local7", LOG_LOCAL7}};

struct s_slog_lvl {
  char *level_in_conf;
  int level;
} sslog_lvl[] = {{"emerg",   LOG_EMERG}, \
		 {"alert",   LOG_ALERT}, \
		 {"crit",    LOG_CRIT}, \
		 {"err",     LOG_ERR}, \
		 {"warning", LOG_WARNING}, \
		 {"notice",  LOG_NOTICE}, \
		 {"info",    LOG_INFO}, \
		 {"debug",   LOG_DEBUG}};

int
read_conf(file)
    char *file;
{
    FILE *fd;     /* Our file descriptor */
    char c[256];  /* it will contains lines of you configuration file */
    char *buf_line = (char *)malloc(256);
    int i;
    
    if (!(fd = fopen(file, "r")))
	return(-1); /* we can't configuration file */
    
    /* init some variables */
    maxtry = 3;
    maxsession = 3;
    maxlogintime = 300;
    secret = premotd = motdpass = motdfail = shell = directory = server = NULL;
    
    /* fill default variable */
    slog = 0;               /* don't use syslog by default */
    slog_flty = 8;          /* facility of syslog by default: local7 */
    slog_lvl = 7;           /* level of syslog by default: info */
    
    while (fgets(c, 256, fd)) {
	if (c[0] == '#') /* we ignore comments lines */
	    continue;
	if (!strncmp("DIE", c, 3)) { /* for lamers */
	    fprintf(stderr, "You have to make your configuration in the "CONF_FILE" file. Fatal.\n");
	    exit(-1);
	}
	for (i = 0; i < MAX_OPTIONSLINE; i++)
	    if (!strncmp(line_conf[i].options, c, strlen(line_conf[i].options))) {
		buf_line = c + strlen(line_conf[i].options);
		switch (line_conf[i].seq) {
		case 1: /* server */
		    if (!server) { /* if we don't have any server yet */
			server = (char *)malloc(strlen(buf_line) - 1);
			strncpy(server, buf_line, strlen(buf_line) - 1);  /* -1 for the '\n' at the end */
		    }
		    break;
		case 2: /* secret */
		    if (!secret) { /* same secret key for all servers */
			secret = (char *)malloc(strlen(buf_line));
			strncpy(secret, buf_line, strlen(buf_line) - 1); /* we don't want the \0 */
		    }
		    break;
		case 3: /* syslog */
		    if (!strncmp(buf_line, "on", 2)) slog = 1; // ok, syslog will be active after the next positive return
		    else if (!strncmp(buf_line, "off", 3)) slog = 0; /* no syslog */
		    break;
		case 4: /* syslog_facility */
		{
		    int j;
		    for (j = 0; j < 8; j++)
			if (!strncmp(buf_line, sslog_flty[j].facility_in_conf, strlen(buf_line) - 1))
			    slog_flty = sslog_flty[j].facility;
		}
		break;
		case 5: /* syslog_level */
		{
		    int j;
		    for (j = 0; j < 8; j++)
			if (!strncmp(buf_line, sslog_lvl[j].level_in_conf, strlen(buf_line) - 1))
			    slog_lvl = sslog_lvl[j].level;
		}
		break;
		case 6: /* premotd */
		    if (premotd)
			fprintf(stderr, "premotd keyword already defined: ignored.\n");
		    else {
			premotd = (char *)malloc(strlen(buf_line));
			strncpy(premotd, buf_line, strlen(buf_line) - 1); /* we don't want the \0 */
			if (file_exist(premotd) < 0) {
			    fprintf(stderr, "PREMOTD: File %s doesn't exist.\n", premotd);
			    premotd = NULL;
			}
		    }
		    break;
		case 7: /* motdpass */
		    if (motdpass)
			fprintf(stderr, "motdpass keyword already defined: ignored.\n");
		    else {
			motdpass = (char *)malloc(strlen(buf_line));
			strncpy(motdpass, buf_line, strlen(buf_line) - 1); /* we don't want the \0 */
			if (file_exist(motdpass) < 0) {
			    fprintf(stderr, "MOTDPASS: File %s doesn't exist.\n", motdpass);
			    motdpass = NULL;
			}
		    }
		    break;
		case 8: /* motdfail */
		    if (motdfail)
			fprintf(stderr, "motdfail keyword already defined: ignored.\n");
		    else {
			motdfail = (char *)malloc(strlen(buf_line));
			strncpy(motdfail, buf_line, strlen(buf_line) - 1); /* we don't want the \0 */
			if (file_exist(motdfail) < 0) {
			    fprintf(stderr, "MOTDFAIL: File %s doesn't exist.\n", motdfail);
			    motdpass = NULL;
			}
		    }
		    break;
		case 9: /* shell */
		    if (!shell) {
			shell = (char *)malloc(strlen(buf_line) - 1);
			strncpy(shell, buf_line, strlen(buf_line) - 1);
			if (file_exist(shell) < 0) {
			    free(shell);
			    shell = NULL;
			}
		    }
		break;
		case 10: /* maxtry */
		    maxtry = atoi(buf_line);
		    break;
		case 11: /* maxsession */
		    maxsession = atoi(buf_line);
		    break;
		case 12: /* directory */
/*		    if (directory) fprintf(stderr, "directory keyword already defined: ignored.\n");
		    else {*/
		        directory = (char *)malloc(strlen(buf_line));
			strncpy(directory, buf_line, strlen(buf_line) - 1); /* we don't want the \0 */
/*			if (file_exist(directory) < 0) {
			    fprintf(stderr, "Directory %s doesn't exist.\n", directory);
			    directory = NULL;
			    }
		    }*/
		    break;
		case 13: /* umask */
		    /* not implemented yet */
		    break;
		case 14: /* passwd_care */
		    if (!strncmp(buf_line, "on", 2)) passwd_care = 1;
		    else if (!strncmp(buf_line, "off", 3)) passwd_care = 0;
		    break;
		case 15: /* maxlogintime */
		    maxlogintime = atoi(buf_line);
		    break;
		case 16: /* uid */
		    uid_default = atoi(buf_line);
		    break;
		case 17: /* gid */
		    gid_default = atoi(buf_line);
		    break;
		default:
		    break;
		}
	    }
    } /* while (.. */
    
    /* did we find any server in the configuration file ? */
    if (!server && !passwd_care) {
	fprintf(stderr, "No server defined in the configuration file '%s'. Goodbye.\n", file);
	exit(-1);
    }

    /* did we find a shell path ? */
    if ((!shell) && (passwd_care == 0)) {
	fprintf(stderr, "No valid shell defined in the configuration file '%s'. Goodbye.\n", file);
	exit(-1);
    }

    /* are uid and gid set ? */
    if (!uid_default && !gid_default) {
	fprintf(stderr, "No valid uid/gid defined in the configuration file '%s'. Goodbye.\n", file);
	exit(-1);
    }
    _syslog = (slog_flty | slog_lvl);

    fclose(fd); /* All variables are init. We don't need the file descriptor anymore. */
    
    return(1);
}
